@@ -216,7 +216,7 @@ INLINE char& operator >>=(char& a,  const Char REF  b)
 
 #endif
 
-#ifdef NO_NRV
+#ifdef _G_NO_NRV
 
 INLINE Char  operator -  (const Char REF  a) 
 { Char r(a); r.negate(); return r; }
