@@ -47,19 +47,20 @@ class BaseSLList {
   protected:
     BaseSLNode *last;
     virtual void delete_node(BaseSLNode*node) = 0;
-    virtual BaseSLNode* copy_node(void* datum) = 0;
+    virtual BaseSLNode* copy_node(const void* datum) = 0;
     virtual void copy_item(void *dst, void *src) = 0;
     virtual ~BaseSLList() { }
     BaseSLList() { last = 0; }
     void copy(const BaseSLList&);
     BaseSLList& operator = (const BaseSLList& a);
-    Pix ins_after(Pix p, void *datum);
-    Pix prepend(void *datum);
-    Pix append(void *datum);
+    Pix ins_after(Pix p, const void *datum);
+    Pix prepend(const void *datum);
+    Pix append(const void *datum);
     int remove_front(void *dst, int signal_error = 0);
     void join(BaseSLList&);
   public:
-    int length();
+    int length() const;
+    int empty() const { return last == 0; }
     void clear();
     Pix                   prepend(BaseSLNode*);
     Pix                   append(BaseSLNode*);
@@ -75,8 +76,8 @@ class SLList : public BaseSLList
 {
   private:
     virtual void delete_node(BaseSLNode *node) { delete (SLNode<T>*)node; }
-    virtual BaseSLNode* copy_node(void *datum)
-	{ return new SLNode<T>(*(T*)datum); }
+    virtual BaseSLNode* copy_node(const void *datum)
+	{ return new SLNode<T>(*(const T*)datum); }
     virtual void copy_item(void *dst, void *src) { *(T*)dst = *(T*)src; }
 
 public:
@@ -86,10 +87,8 @@ public:
 	{ BaseSLList::operator=((const BaseSLList&) a); return *this; }
     virtual ~SLList() { clear(); }
 
-    int                   empty() { return last == 0; }
-
-    Pix prepend(T& item) {return BaseSLList::prepend(&item);}
-    Pix append(T& item) {return BaseSLList::append(&item);}
+    Pix prepend(const T& item) {return BaseSLList::prepend(&item);}
+    Pix append(const T& item) {return BaseSLList::append(&item);}
     Pix prepend(SLNode<T>* node) {return BaseSLList::prepend(node);}
     Pix append(SLNode<T>* node) {return BaseSLList::append(node);}
 
@@ -99,7 +98,8 @@ public:
     inline Pix first() { return (last == 0)? 0 : Pix(last->tl); }
     void next(Pix& p)
 	{ p = (p == 0 || p == last)? 0 : Pix(((SLNode<T>*)(p))->tl); }
-    Pix ins_after(Pix p, T& item) { return BaseSLList::ins_after(p, &item); }
+    Pix ins_after(Pix p, const T& item)
+      { return BaseSLList::ins_after(p, &item); }
     void join(SLList<T>& a) { BaseSLList::join(a); }
     
     T& front() {
