@@ -193,7 +193,7 @@ StrRep* Sresize(StrRep* old, int newlen)
 
 // like allocate, but we know that src is a StrRep
 
-StrRep* Scopy(StrRep* old, StrRep* s)
+StrRep* Scopy(StrRep* old, const StrRep* s)
 {
   if (old == &_nilStrRep) old = 0;
   if (s == &_nilStrRep) s = 0;
@@ -483,7 +483,7 @@ int String::match(int start, int sl, int exact, const char* t, int tl) const
   return tl;
 }
 
-void SubString::assign(StrRep* ysrc, const char* ys, int ylen)
+void SubString::assign(const StrRep* ysrc, const char* ys, int ylen)
 {
   if (&S == &_nilString) return;
 
@@ -959,7 +959,7 @@ int split(const String& src, String results[], int n, const Regex& r)
 }
 
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 String join(String src[], int n, const String& separator) return x;
 {
@@ -1016,7 +1016,7 @@ String join(String src[], int n, const String& separator)
 */
 
     
-StrRep* Sreverse(StrRep* src, StrRep* dest)
+StrRep* Sreverse(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest)
@@ -1036,7 +1036,7 @@ StrRep* Sreverse(StrRep* src, StrRep* dest)
 }
 
 
-StrRep* Supcase(StrRep* src, StrRep* dest)
+StrRep* Supcase(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1046,7 +1046,7 @@ StrRep* Supcase(StrRep* src, StrRep* dest)
   return dest;
 }
 
-StrRep* Sdowncase(StrRep* src, StrRep* dest)
+StrRep* Sdowncase(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1056,7 +1056,7 @@ StrRep* Sdowncase(StrRep* src, StrRep* dest)
   return dest;
 }
 
-StrRep* Scapitalize(StrRep* src, StrRep* dest)
+StrRep* Scapitalize(const StrRep* src, StrRep* dest)
 {
   int n = src->len;
   if (src != dest) dest = Salloc(dest, src->s, n, n);
@@ -1077,7 +1077,9 @@ StrRep* Scapitalize(StrRep* src, StrRep* dest)
       {
         if (isupper(*p))
           *p = tolower(*p);
-        else if (!islower(*p) && !isdigit(*p))
+	/* A '\'' does not break a word, so that "Nathan's" stays
+	   "Nathan's" rather than turning into "Nathan'S". */
+        else if (!islower(*p) && !isdigit(*p) && (*p != '\''))
           break;
       }
     }
@@ -1085,7 +1087,7 @@ StrRep* Scapitalize(StrRep* src, StrRep* dest)
   return dest;
 }
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 String replicate(char c, int n) return w;
 {
@@ -1195,23 +1197,11 @@ String common_suffix(const String& x, const String& y, int startpos)
 
 istream& operator>>(istream& s, String& x)
 {
-#ifdef _OLD_STREAMS
-  if (!s.good())
-  {
-    return s;
-  }
-  s >> ws;
-  if (!s.good())
-  {
-    return s;
-  }
-#else
   if (!s.ipfx(0) || (!(s.flags() & ios::skipws) && !ws(s)))
   {
-    s.set(ios::failbit); // Redundant if using GNU iostreams.
+    s.clear(ios::failbit|s.rdstate()); // Redundant if using GNU iostreams.
     return s;
   }
-#endif
   int ch;
   int i = 0;
   x.rep = Sresize(x.rep, 20);
@@ -1226,21 +1216,17 @@ istream& operator>>(istream& s, String& x)
   }
   x.rep->s[i] = 0;
   x.rep->len = i;
-  if (i == 0) s.set(_fail);
-  if (ch == EOF) s.set(_eof);
+  int new_state = s.rdstate();
+  if (i == 0) new_state |= ios::failbit;
+  if (ch == EOF) new_state |= ios::eofbit;
+  s.clear(new_state);
   return s;
 }
 
 int readline(istream& s, String& x, char terminator, int discard)
 {
-#ifdef _OLD_STREAMS
-  if (!s.good())
-#else
   if (!s.ipfx(0))
-#endif
-  {
     return 0;
-  }
   int ch;
   int i = 0;
   x.rep = Sresize(x.rep, 80);
@@ -1258,7 +1244,7 @@ int readline(istream& s, String& x, char terminator, int discard)
   }
   x.rep->s[i] = 0;
   x.rep->len = i;
-  if (ch == EOF) s.set(_eof);
+  if (ch == EOF) s.clear(ios::eofbit|s.rdstate());
   return i;
 }
 
@@ -1318,7 +1304,7 @@ int String::OK() const
 
 int SubString::OK() const
 {
-  int v = S != 0;               // have a String;
+  int v = S != (const char*)0; // have a String;
   v &= S.OK();                 // that is legal
   v &= pos + len >= S.rep->len;// pos and len within bounds
   if (!v) S.error("SubString invariant failure");
