@@ -38,15 +38,15 @@ struct StrRep                     // internal String representations
 // primitive ops on StrReps -- nearly all String fns go through these.
 
 StrRep*     Salloc(StrRep*, const char*, int, int);
-StrRep*     Scopy(StrRep*, StrRep*);
+StrRep*     Scopy(StrRep*, const StrRep*);
 StrRep*     Sresize(StrRep*, int);
 StrRep*     Scat(StrRep*, const char*, int, const char*, int);
 StrRep*     Scat(StrRep*, const char*, int,const char*,int, const char*,int);
 StrRep*     Sprepend(StrRep*, const char*, int);
-StrRep*     Sreverse(StrRep*, StrRep*);
-StrRep*     Supcase(StrRep*, StrRep*);
-StrRep*     Sdowncase(StrRep*, StrRep*);
-StrRep*     Scapitalize(StrRep*, StrRep*);
+StrRep*     Sreverse(const StrRep*, StrRep*);
+StrRep*     Supcase(const StrRep*, StrRep*);
+StrRep*     Sdowncase(const StrRep*, StrRep*);
+StrRep*     Scapitalize(const StrRep*, StrRep*);
 
 // These classes need to be defined in the order given
 
@@ -62,7 +62,7 @@ protected:
   unsigned short    pos;      // starting position in S's rep
   unsigned short    len;      // length of substring
 
-  void              assign(StrRep*, const char*, int = -1);
+  void              assign(const StrRep*, const char*, int = -1);
                     SubString(String& x, int p, int l);
                     SubString(const SubString& x);
 
@@ -73,10 +73,10 @@ public:
 
                    ~SubString();
 
-  void              operator =  (const String&     y);
-  void              operator =  (const SubString&  y);
-  void              operator =  (const char* t);
-  void              operator =  (char        c);
+  SubString&        operator =  (const String&     y);
+  SubString&        operator =  (const SubString&  y);
+  SubString&        operator =  (const char* t);
+  SubString&        operator =  (char        c);
 
 // return 1 if target appears anywhere in SubString; else 0
 
@@ -134,17 +134,17 @@ public:
 
                     ~String();
 
-  void              operator =  (const String&     y);
-  void              operator =  (const char* y);
-  void              operator =  (char        c);
-  void              operator =  (const SubString&  y);
+  String&           operator =  (const String&     y);
+  String&           operator =  (const char* y);
+  String&           operator =  (char        c);
+  String&           operator =  (const SubString&  y);
 
 // concatenation
 
-  void              operator += (const String&     y); 
-  void              operator += (const SubString&  y);
-  void              operator += (const char* t);
-  void              operator += (char        c);
+  String&           operator += (const String&     y); 
+  String&           operator += (const SubString&  y);
+  String&           operator += (const char* t);
+  String&           operator += (char        c);
 
   void              prepend(const String&     y); 
   void              prepend(const SubString&  y);
@@ -485,45 +485,53 @@ inline SubString::~SubString() {}
 
 // assignment
 
-inline void String::operator =  (const String& y)
+inline String& String::operator =  (const String& y)
 { 
   rep = Scopy(rep, y.rep);
+  return *this;
 }
 
-inline void String::operator=(const char* t)
+inline String& String::operator=(const char* t)
 {
-  rep = Salloc(rep, t, -1, -1); 
+  rep = Salloc(rep, t, -1, -1);
+  return *this;
 }
 
-inline void String::operator=(const SubString&  y)
+inline String& String::operator=(const SubString&  y)
 {
   rep = Salloc(rep, y.chars(), y.length(), y.length());
+  return *this;
 }
 
-inline void String::operator=(char c)
+inline String& String::operator=(char c)
 {
-  rep = Salloc(rep, &c, 1, 1); 
+  rep = Salloc(rep, &c, 1, 1);
+  return *this;
 }
 
 
-inline void SubString::operator = (const char* ys)
+inline SubString& SubString::operator = (const char* ys)
 {
   assign(0, ys);
+  return *this;
 }
 
-inline void SubString::operator = (char ch)
+inline SubString& SubString::operator = (char ch)
 {
   assign(0, &ch, 1);
+  return *this;
 }
 
-inline void SubString::operator = (const String& y)
+inline SubString& SubString::operator = (const String& y)
 {
   assign(y.rep, y.chars(), y.length());
+  return *this;
 }
 
-inline void SubString::operator = (const SubString& y)
+inline SubString& SubString::operator = (const SubString& y)
 {
   assign(y.S.rep, y.chars(), y.length());
+  return *this;
 }
 
 // Zillions of cats...
@@ -712,29 +720,33 @@ inline void cat(const char* a, const char* x, char y, String& r)
 
 // operator versions
 
-inline void String::operator +=(const String& y)
+inline String& String::operator +=(const String& y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
-inline void String::operator +=(const SubString& y)
+inline String& String::operator +=(const SubString& y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
-inline void String::operator += (const char* y)
+inline String& String::operator += (const char* y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
-inline void String:: operator +=(char y)
+inline String& String:: operator +=(char y)
 {
   cat(*this, y, *this);
+  return *this;
 }
 
 // constructive concatenation
 
-#if defined(__GNUG__) && !defined(NO_NRV)
+#if defined(__GNUG__) && !defined(_G_NO_NRV)
 
 inline String operator + (const String& x, const String& y) return r;
 {
