@@ -8,12 +8,12 @@ TSRCS =  tObstack.cc tString.cc tInteger.cc tRational.cc \
  tiLList.cc
 EOF
 
-TESTS1="tObstack tString tInteger tRational tComplex tBitSet tBitString tStack tQueue tDeque tPQ tSet tBag tMap tFix tFix16 tFix24 tList tPlex tLList tVec"
-TTESTS="tRandom ${TESTS1} tiLList"
+TESTS0="tObstack tString tInteger tRational tComplex tBitSet tBitString tRandom"
+TESTS1="tStack tQueue tDeque tPQ tSet tBag tMap tFix tFix16 tFix24 tList tPlex tLList tVec"
 
 cat <<EOF
 # executables
-TOUTS =  test_h tCurses ${TTESTS} tGetOpt
+TOUTS =  test_h tCurses ${TESTS0} ${TESTS1} tiLList tGetOpt
 
 EOF
 
@@ -63,7 +63,7 @@ check: tests
 .PHONY: check-tGetOpt
 EOF
 
-for TEST in tRandom ${TESTS1} ; do
+for TEST in ${TESTS0} ${TESTS1} ; do
   echo ".PHONY: check-${TEST}"
   echo "check-${TEST}: ${TEST}" '$(srcdir)'"/${TEST}.inp"
   echo "	./${TEST} < "'$(srcdir)'"/${TEST}.inp > ${TEST}.out 2>&1"
@@ -133,7 +133,13 @@ LIB_FOR_tFix16=-lm
 LIB_FOR_tFix24=-lm
 LIB_FOR_tCurses="-lcurses -ltermcap"
 
-for TEST in twrapper tgwrapper $TTESTS tCurses tGetOpt; do
+for TEST in $TESTS0 tiLList; do
+  echo "${TEST}: ${TEST}.o"
+  echo '	$(CXX) $(LDFLAGS)' "${TEST}.o" '-o $@ $(LIBS)' \
+	`eval echo '$LIB_FOR_'$TEST`
+  echo ""
+done
+for TEST in twrapper tgwrapper $TESTS1 tCurses tGetOpt; do
   echo "${TEST}: " '$(LIBTEST)' " ${TEST}.o"
   echo '	$(CXX) $(LDFLAGS)' "${TEST}.o" '-o $@ $(LIBTEST) $(LIBS)' \
 	`eval echo '$LIB_FOR_'$TEST`
