@@ -300,6 +300,10 @@ void utiltest()
   cout << "capitalize(x) = " << z << "\n";
   assert(z.OK());
   assert(z == "Hello");
+
+  /* Let's see how apostrophe is handled. */
+  z = "he asked:'this is nathan's book?'. 'no, it's not',i said.";
+  cout << "capitalize(z) = " << capitalize (z) << "\n";
   
   z = replicate('*', 10);
   cout << "z = replicate(*, 10) = " << z << "\n";
