@@ -88,6 +88,7 @@ _IO_file_init(fp)
      register _IO_FILE *fp;
 {
   fp->_offset = _IO_pos_0;
+  fp->_IO_file_flags |= CLOSED_FILEBUF_FLAGS;
 
   _IO_link_in(fp);
   fp->_fileno = -1;
@@ -101,10 +102,7 @@ _IO_file_close_it(fp)
   if (!_IO_file_is_open(fp))
     return EOF;
 
-  /* This flushes as well as switching mode. */
-  if (fp->_IO_write_ptr > fp->_IO_write_base || _IO_in_put_mode(fp))
-    if (_IO_switch_to_get_mode(fp))
-	  return EOF;
+  _IO_do_flush (fp);
 
   _IO_unsave_markers(fp);
 
@@ -127,8 +125,12 @@ void
 _IO_file_finish(fp)
      register _IO_FILE* fp;
 {
-  if (!(fp->_flags & _IO_DELETE_DONT_CLOSE))
-    _IO_file_close_it(fp);
+  if (_IO_file_is_open(fp))
+    {
+      _IO_do_flush (fp);
+      if (!(fp->_flags & _IO_DELETE_DONT_CLOSE))
+	fp->_jumps->__close(fp);
+    }
   _IO_default_finish(fp);
 }
 
@@ -170,8 +172,7 @@ _IO_file_fopen(fp, filename, mode)
   if (fdesc < 0)
     return NULL;
   fp->_fileno = fdesc;
-  fp->_IO_file_flags = 
-    _IO_mask_flags(fp, read_write,_IO_NO_READS+_IO_NO_WRITES+_IO_IS_APPENDING);
+  _IO_mask_flags(fp, read_write,_IO_NO_READS+_IO_NO_WRITES+_IO_IS_APPENDING);
   if (read_write & _IO_IS_APPENDING)
     if (fp->_jumps->__seekoff(fp, (_IO_off_t)0, _IO_seek_end) == _IO_pos_BAD)
       return NULL;
@@ -188,6 +189,7 @@ _IO_file_attach(fp, fd)
     return NULL;
   fp->_fileno = fd;
   fp->_flags &= ~(_IO_NO_READS+_IO_NO_WRITES);
+  fp->_flags |= _IO_DELETE_DONT_CLOSE;
   fp->_offset = _IO_pos_BAD;
   return fp;
 }
@@ -245,7 +247,7 @@ int
 _IO_file_underflow(fp)
      register _IO_FILE *fp;
 {
-  _IO_size_t count;
+  _IO_ssize_t count;
 #if 0
   /* SysV does not make this test; take it out for compatibility */
   if (fp->_flags & _IO_EOF_SEEN)
@@ -504,7 +506,7 @@ _IO_file_read(fp, buf, size)
     {
       _IO_ssize_t count = _IO_read(fp->_fileno, buf, size);
 #ifdef EINTR
-      if (errno == EINTR && count == -1)
+      if (count == -1 && errno == EINTR)
 	continue;
 #endif
       return count;
@@ -636,13 +638,82 @@ _IO_file_xsputn(f, data, n)
       
       /* Now write out the remainder.  Normally, this will fit in the
 	 buffer, but it's somewhat messier for line-buffered files,
-	 so we let _IO_xsputn handle the general case. */
+	 so we let _IO_default_xsputn handle the general case. */
       if (dont_write)
 	to_do -= _IO_default_xsputn(f, s+count, dont_write);
     }
   return n - to_do;
 }
 
+#if 0
+/* Work in progress */
+_IO_size_t
+_IO_file_xsgetn(fp, data, n)
+     _IO_FILE *fp;
+     void *data;
+     _IO_size_t n;
+{
+  register _IO_size_t more = n;
+  register char *s = data;
+  for (;;)
+    {
+      _IO_ssize_t count = fp->_IO_read_end - fp->_IO_read_ptr; /* Data available. */
+      if (count > 0)
+	{
+	  if (count > more)
+	    count = more;
+	  if (count > 20)
+	    {
+	      memcpy(s, fp->_IO_read_ptr, count);
+	      s += count;
+	      fp->_IO_read_ptr += count;
+	    }
+	  else if (count <= 0)
+	    count = 0;
+	  else
+	    {
+	      register char *p = fp->_IO_read_ptr;
+	      register int i = (int)count;
+	      while (--i >= 0) *s++ = *p++;
+	      fp->_IO_read_ptr = p;
+            }
+            more -= count;
+        }
+#if 0
+      if (! _IO_in put_mode (fp)
+	  && ! _IO_have_markers (fp) && ! IO_have_backup (fp))
+	{
+	  /* This is an optimization of _IO_file_underflow */
+	  if (fp->_flags & _IO_NO_READS)
+	    break;
+	  /* If we're reading a lot of data, don't bother allocating
+	     a buffer.  But if we're only reading a bit, perhaps we should ??*/
+	  if (count <= 512 && fp->_IO_buf_base == NULL)
+	    _IO_doallocbuf(fp);
+	  if (fp->_flags & (_IO_LINE_BUF|_IO_UNBUFFERED))
+	    _IO_flush_all_linebuffered();
+
+	  _IO_switch_to_get_mode(fp); ???;
+	  count = fp->_jumps->__read (fp, s, more);
+	  if (count <= 0)
+	     {
+	       if (count == 0)
+		 fp->_flags |= _IO_EOF_SEEN;
+	       else
+		 fp->_flags |= _IO_ERR_SEEN, count = 0;
+	     }
+	  
+	  s += count;
+	  more -= count;
+	}
+#endif
+      if (more == 0 || __underflow(fp) == EOF)
+	break;
+    }
+  return n - more;
+}
+#endif
+
 struct _IO_jump_t _IO_file_jumps = {
   _IO_file_overflow,
   _IO_file_underflow,
@@ -660,4 +731,5 @@ struct _IO_jump_t _IO_file_jumps = {
   _IO_file_seek,
   _IO_file_seekoff,
   _IO_default_seekpos,
+  _IO_default_uflow
 };
