@@ -125,6 +125,7 @@ the executable file might be covered by the GNU General Public License. */
 /* In this case, we assume IEEE floats. */
 #define FLT_ROUNDS 1
 #define FLT_RADIX 2
+#define DBL_MANT_DIG 53
 #define DBL_DIG 15
 #define DBL_MAX_10_EXP 308
 #define DBL_MAX_EXP 1024
@@ -141,10 +142,12 @@ the executable file might be covered by the GNU General Public License. */
 #define Sign_Extend(a,b) /*no-op*/
 #endif
 
-#if defined(__i386__) || defined(clipper) || defined(MIPSEL)
+#if defined(__i386__) || defined(__i860__) || defined(clipper)
+#define IEEE_8087
+#endif
+#if defined(MIPSEL) || defined(__alpha__)
 #define IEEE_8087
 #endif
-
 #if defined(__sparc__) || defined(sparc) || defined(MIPSEB)
 #define IEEE_MC68k
 #endif
@@ -153,20 +156,27 @@ the executable file might be covered by the GNU General Public License. */
 
 #if FLT_RADIX==16
 #define IBM
-#elif DBL_MANT_DIG==56
+#else
+#if DBL_MANT_DIG==56
 #define VAX
-#elif DBL_MANT_DIG==53 && DBL_MAX_10_EXP==308
+#else
+#if DBL_MANT_DIG==53 && DBL_MAX_10_EXP==308
 #define IEEE_Unknown
 #else
 Exactly one of IEEE_8087, IEEE_MC68k, VAX, or IBM should be defined.
 #endif
 #endif
+#endif
+#endif
+
+typedef _G_uint32_t unsigned32;
 
 #ifdef IEEE_8087
 #define HIWORD 1
 #define LOWORD 0
 #define TEST_ENDIANNESS  /* nothing */
-#elif defined(IEEE_MC68k)
+#else
+#if defined(IEEE_MC68k)
 #define HIWORD 0
 #define LOWORD 1
 #define TEST_ENDIANNESS  /* nothing */
@@ -176,7 +186,7 @@ static void test_endianness()
 {
     union doubleword {
 	double d;
-	unsigned long u[2];
+	unsigned32 u[2];
     } dw;
     dw.d = 10;
     if (dw.u[0] != 0) /* big-endian */
@@ -186,29 +196,33 @@ static void test_endianness()
 }
 #define TEST_ENDIANNESS  if (HIWORD<0) test_endianness();
 #endif
+#endif
+
 #if 0
 union {
   double d;
-  unsigned long x[2];
+  unsigned32 x[2];
 } _temp;
 #endif
-#define word0(x) ((unsigned long *)&x)[HIWORD]
+#define word0(x) ((unsigned32 *)&x)[HIWORD]
 #if 0
 #define word0(X) (_temp.d = X, _temp.x[HIWORD])
 #define setword0(D,X) (_temp.d = D, _temp.x[HIWORD] = X, D = _temp.d)
 #endif
-#define word1(x) ((unsigned long *)&x)[LOWORD]
+#define word1(x) ((unsigned32 *)&x)[LOWORD]
 
 /* The following definition of Storeinc is appropriate for MIPS processors. */
 #if defined(IEEE_8087) + defined(VAX)
 #define Storeinc(a,b,c) (((unsigned short *)a)[1] = (unsigned short)b, \
 ((unsigned short *)a)[0] = (unsigned short)c, a++)
-#elif defined(IEEE_MC68k)
+#else
+#if defined(IEEE_MC68k)
 #define Storeinc(a,b,c) (((unsigned short *)a)[0] = (unsigned short)b, \
 ((unsigned short *)a)[1] = (unsigned short)c, a++)
 #else
 #define Storeinc(a,b,c) (*a++ = b << 16 | c & 0xffff)
 #endif
+#endif
 
 /* #define P DBL_MANT_DIG */
 /* Ten_pmax = floor(P*log(2)/log(5)) */
@@ -328,17 +342,17 @@ struct Bigint {
   short on_stack;	/* 1 if stack-allocated. */
   short sign;		/* 0 if value is positive or zero; 1 if negative. */
   int wds;		/* Current length. */
-  unsigned long x[1<<BIGINT_MINIMUM_K]; /* Actually: x[maxwds] */
+  unsigned32 x[1<<BIGINT_MINIMUM_K]; /* Actually: x[maxwds] */
 };
 
 #define BIGINT_HEADER_SIZE \
-  (sizeof(Bigint) - (1<<BIGINT_MINIMUM_K) * sizeof(unsigned long))
+  (sizeof(Bigint) - (1<<BIGINT_MINIMUM_K) * sizeof(unsigned32))
 
 typedef struct Bigint Bigint;
 
 /* Initialize a stack-allocated Bigint. */
 
-Bigint *
+static Bigint *
 Binit
 #ifdef KR_headers
         (v) Bigint *v;
@@ -371,7 +385,7 @@ Balloc
 
   x = 1 << k;
   rv = (Bigint *)
-    malloc(BIGINT_HEADER_SIZE + x * sizeof(unsigned long));
+    malloc(BIGINT_HEADER_SIZE + x * sizeof(unsigned32));
   rv->k = k;
   rv->maxwds = x;
   rv->sign = rv->wds = 0;
@@ -399,7 +413,7 @@ Bcopy
         (Bigint *x, Bigint *y)
 #endif
 {
-  register unsigned long *xp, *yp;
+  register unsigned32 *xp, *yp;
   register int i = y->wds;
   x->sign = y->sign;
   x->wds = i;
@@ -442,8 +456,8 @@ multadd
 #endif
 {
         int i, wds;
-        unsigned long *x, y;
-        unsigned long xi, z;
+        unsigned32 *x, y;
+        unsigned32 xi, z;
 
         wds = b->wds;
         x = b->x;
@@ -469,9 +483,9 @@ static Bigint *
 s2b
 #ifdef KR_headers
         (result, s, nd0, nd, y9)
-	Bigint *result; CONST char *s; int nd0, nd; unsigned long y9;
+	Bigint *result; CONST char *s; int nd0, nd; unsigned32 y9;
 #else
-        (Bigint *result, CONST char *s, int nd0, int nd, unsigned long y9)
+        (Bigint *result, CONST char *s, int nd0, int nd, unsigned32 y9)
 #endif
 {
   int i, k;
@@ -502,9 +516,9 @@ s2b
 static int
 hi0bits
 #ifdef KR_headers
-        (x) register unsigned long x;
+        (x) register unsigned32 x;
 #else
-        (register unsigned long x)
+        (register unsigned32 x)
 #endif
 {
         register int k = 0;
@@ -536,13 +550,13 @@ hi0bits
 static int
 lo0bits
 #ifdef KR_headers
-        (y) unsigned long *y;
+        (y) unsigned32 *y;
 #else
-        (unsigned long *y)
+        (unsigned32 *y)
 #endif
 {
         register int k;
-        register unsigned long x = *y;
+        register unsigned32 x = *y;
 
         if (x & 7) {
                 if (x & 1)
@@ -606,9 +620,9 @@ mult
 #endif
 {
         int k, wa, wb, wc;
-        unsigned long carry, y, z;
-        unsigned long *x, *xa, *xae, *xb, *xbe, *xc, *xc0;
-        unsigned long z2;
+        unsigned32 carry, y, z;
+        unsigned32 *x, *xa, *xae, *xb, *xbe, *xc, *xc0;
+        unsigned32 z2;
         if (a->wds < b->wds) {
                 Bigint *tmp = a;
                 a = b;
@@ -696,7 +710,7 @@ lshift
 #endif
 {
   int i;
-  unsigned long *x, *x1, *xe;
+  unsigned32 *x, *x1, *xe;
   int old_wds = b->wds;
   int n = k >> 5;
   int k1 = b->k;
@@ -714,12 +728,12 @@ lshift
   x1 = x + n; /* Destination pointer */
   if (k &= 0x1f) {
     int k1 = 32 - k;
-    unsigned long z = *--x;
+    unsigned32 z = *--x;
     if ((*x1 = (z >> k1)) != 0) {
       ++n1;
     }
     while (x > xe) {
-      unsigned long w = *--x;
+      unsigned32 w = *--x;
       *--x1 = (z << k) | (w >> k1);
       z = w;
     }
@@ -743,7 +757,7 @@ cmp
         (Bigint *a, Bigint *b)
 #endif
 {
-        unsigned long *xa, *xa0, *xb, *xb0;
+        unsigned32 *xa, *xa0, *xb, *xb0;
         int i, j;
 
         i = a->wds;
@@ -781,7 +795,7 @@ diff
 {
         int i, wa, wb;
         long borrow, y; /* We need signed shifts here. */
-        unsigned long *xa, *xae, *xb, *xbe, *xc;
+        unsigned32 *xa, *xae, *xb, *xbe, *xc;
         long z;
 
         i = cmp(a,b);
@@ -880,11 +894,11 @@ b2d
         (Bigint *a, int *e)
 #endif
 {
-        unsigned long *xa, *xa0, w, y, z;
+        unsigned32 *xa, *xa0, w, y, z;
         int k;
         double d;
 #ifdef VAX
-        unsigned long d0, d1;
+        unsigned32 d0, d1;
 #else
 #define d0 word0(d)
 #define d1 word1(d)
@@ -934,9 +948,9 @@ d2b
 #endif
 {
         int de, i, k;
-        unsigned long *x, y, z;
+        unsigned32 *x, y, z;
 #ifdef VAX
-        unsigned long d0, d1;
+        unsigned32 d0, d1;
         d0 = word0(d) >> 16 | word0(d) << 16;
         d1 = word1(d) >> 16 | word1(d) << 16;
 #else
@@ -1078,7 +1092,7 @@ _IO_strtod
         CONST char *s, *s0, *s1;
         double aadj, aadj1, adj, rv, rv0;
         long L;
-        unsigned long y, z;
+        unsigned32 y, z;
 	Bigint _bb, _b_avail, _bd, _bd0, _bs, _delta;
 	Bigint *bb = Binit(&_bb);
 	Bigint *bd = Binit(&_bd);
@@ -1099,6 +1113,9 @@ _IO_strtod
                                 goto break2;
                         /* no break */
                 case 0:
+			/* "+" and "-" should be reported as an error? */
+			sign = 0;
+			s = s00;
                         goto ret;
                 case '\t':
                 case '\n':
@@ -1604,10 +1621,10 @@ quorem
 {
         int n;
         long borrow, y;
-        unsigned long carry, q, ys;
-        unsigned long *bx, *bxe, *sx, *sxe;
+        unsigned32 carry, q, ys;
+        unsigned32 *bx, *bxe, *sx, *sxe;
         long z;
-        unsigned long si, zs;
+        unsigned32 si, zs;
 
         n = S->wds;
 #ifdef DEBUG
@@ -1777,8 +1794,13 @@ _IO_dtoa
 
 	TEST_ENDIANNESS;
         if (result) {
+		/* result is contains a string, so its fields (interpreted
+		   as a Bigint have been trashed.  Restore them.
+		   This is a really ugly interface - result should
+		   not be static, since that is not thread-safe.  FIXME. */
                 result->k = result_k;
                 result->maxwds = 1 << result_k;
+                result->on_stack = 0;
                 }
 
         if (word0(d) & Sign_bit) {
@@ -1849,7 +1871,7 @@ _IO_dtoa
                 }
         else {
                 /* d is denormalized */
-		unsigned long x;
+		unsigned32 x;
 
                 i = bbits + be + (Bias + (P-1) - 1);
                 x = i > 32  ? word0(d) << 64 - i | word1(d) >> i - 32
@@ -1948,11 +1970,15 @@ _IO_dtoa
                                 i = 1;
                 }
 	/* i is now an upper bound of the number of digits to generate. */
-        j = sizeof(unsigned long) * (1<<BIGINT_MINIMUM_K);
+        j = sizeof(unsigned32) * (1<<BIGINT_MINIMUM_K);
 	/* The test is <= so as to allow room for the final '\0'. */
         for(result_k = BIGINT_MINIMUM_K; BIGINT_HEADER_SIZE + j <= i;
                 j <<= 1) result_k++;
-        result = Brealloc(result, result_k);
+        if (!result || result_k > result->k)
+        {
+          Bfree (result);
+          result = Balloc(result_k);
+        }
         s = s0 = (char *)result;
 
         if (ilim >= 0 && ilim <= Quick_max && try_quick) {
