@@ -23,6 +23,9 @@ This exception does not however invalidate any other reasons why
 the executable file might be covered by the GNU General Public License. */
 
 #include "libioP.h"
+#ifdef __STDC__
+#include <stdlib.h>
+#endif
 
 int
 _IO_fclose(fp)
@@ -30,8 +33,15 @@ _IO_fclose(fp)
 {
   int status = 0;
   COERCE_FILE(fp);
-  status = fp->_jumps->__close(fp);
+  if ((fp->_IO_file_flags & _IO_MAGIC_MASK) != _IO_MAGIC)
+    return EOF;
+  if (fp->_IO_file_flags & _IO_IS_FILEBUF)
+    status = _IO_file_close_it(fp);
   fp->_jumps->__finish(fp);
-  free(fp);
+  if (fp != _IO_stdin && fp != _IO_stdout && fp != _IO_stderr)
+    {
+      fp->_IO_file_flags = 0;
+      free(fp);
+    }
   return status;
 }
