@@ -30,10 +30,13 @@ _IO_fgets(buf, n, fp)
      int n;
      _IO_FILE* fp;
 {
-  long count;
+  _IO_size_t count;
   COERCE_FILE(fp);
-  count = _IO_getline(fp, buf, n, '\n', 1);
-  if (count <= 0)
+  if (n <= 0)
     return NULL;
+  count = _IO_getline(fp, buf, n - 1, '\n', 1);
+  if (count == 0 || (fp->_IO_file_flags & _IO_ERR_SEEN))
+    return NULL;
+  buf[count] = 0;
   return buf;
 }
