@@ -69,59 +69,88 @@ static char sccsid[] = "%W% (Berkeley) %G%";
 /* end of configuration stuff */
 
 
-#ifdef TODO
 /*
- * Helper class and function for `fprintf to unbuffered': creates a
- * temporary buffer.  We only work on write-only files; this avoids
- * worries about ungetc buffers and so forth.
- */
+ * Helper "class" for `fprintf to unbuffered': creates a
+ * temporary buffer. */
 
-class help_streambuf : public backupbuf {
-  public:
-    char *buffer;
-    int buf_size;
-    streambuf *sb;
-    help_streambuf(streambuf *sbuf, char *buf, int n) {
-	sb = sbuf; buffer = buf; buf_size = n;
-	setp(buffer, buffer+buf_size); }
-    ~help_streambuf();
-    virtual int overflow(int c = EOF);
+struct helper_file
+{
+  struct _IO_FILE_plus _f;
+  _IO_FILE *_put_stream;
 };
 
-int help_streambuf::overflow(int c)
+static int
+_IO_helper_overflow (fp, c)
+     _IO_FILE *fp;
+     int c;
 {
-    int used = pptr() - pbase();
-    if (used) {
-	sb->sputn(pbase(), used);
-	pbump(-used);
-    }
-    if (c == EOF || buf_size == 0)
-	return sb->overflow(c);
-    return sputc(c);
-}
-help_streambuf::~help_streambuf()
-{
-    int used = pptr() - pbase();
-    if (used) {
-	sb->sputn(pbase(), used);
-	pbump(-used);
+  _IO_FILE *target = ((struct helper_file*)fp)->_put_stream;
+  int used = fp->_IO_write_ptr - fp->_IO_write_base;
+  if (used)
+    {
+      _IO_sputn(target, fp->_IO_write_base, used);
+      fp->_IO_write_ptr -= used;
     }
+  return _IO_putc (c, fp);
 }
 
-int help_vform(streambuf *sb, char const *fmt0, va_list ap)
+static struct _IO_jump_t _IO_helper_jumps = {
+  _IO_helper_overflow,
+  _IO_default_underflow,
+  _IO_default_xsputn,
+  _IO_default_xsgetn,
+  _IO_default_read,
+  _IO_default_write,
+  _IO_default_doallocate,
+  _IO_default_pbackfail,
+  _IO_default_setbuf,
+  _IO_default_sync,
+  _IO_default_finish,
+  _IO_default_close,
+  _IO_default_stat,
+  _IO_default_seek,
+  _IO_default_seekoff,
+  _IO_default_seekpos,
+  _IO_default_uflow
+};
+
+static int
+helper_vfprintf(fp, fmt0, ap)
+     register _IO_FILE* fp;
+     char const *fmt0;
+     _IO_va_list ap;
 {
-    char buf[_IO_BUFSIZ];
-    help_streambuf helper(sb, buf, _IO_BUFSIZ);
-    return helper.vform(fmt0, ap);
+  char buf[_IO_BUFSIZ];
+  struct helper_file helper;
+  register _IO_FILE *hp = (_IO_FILE*)&helper;
+  int result, to_flush;
+
+  /* initialize helper */
+  helper._put_stream = fp;
+  hp->_IO_write_base = buf;
+  hp->_IO_write_ptr = buf;
+  hp->_IO_write_end = buf+_IO_BUFSIZ;
+  hp->_IO_file_flags = _IO_MAGIC|_IO_NO_READS;
+  hp->_jumps = &_IO_helper_jumps;
+  
+  /* Now print to helper instead. */
+  result = _IO_vfprintf(hp, fmt0, ap);
+
+  /* Now flush anything from the helper to the fp. */
+  if ((to_flush = hp->_IO_write_ptr - hp->_IO_write_base) > 0)
+    {
+      if (_IO_sputn(fp, hp->_IO_write_base, to_flush) != to_flush)
+	return EOF;
+    }
+  return result;
 }
-#endif
 
 #ifdef FLOATING_POINT
 
 #include "floatio.h"
 #define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */
 #define	DEFPREC		6
-extern double modf _PARAMS((double, double*));
+extern double modf __P((double, double*));
 
 #else /* no FLOATING_POINT */
 
@@ -204,11 +233,9 @@ _IO_vfprintf(fp, fmt0, ap)
 	    flags&SHORTINT ? (unsigned long)(unsigned short)va_arg(ap, int) : \
 	    (unsigned long)va_arg(ap, unsigned int))
 
-#ifdef TODO
-	/* optimise cerr (and other unbuffered Unix files) */
-	if (unbuffered())
-	    return help_vform(this, fmt0, ap);
-#endif
+	/* optimise stderr (and other unbuffered Unix files) */
+	if (fp->_IO_file_flags & _IO_UNBUFFERED)
+	    return helper_vfprintf(fp, fmt0, ap);
 
 	fmt = fmt0;
 	ret = 0;
