@@ -1,5 +1,5 @@
-#include <stdio.h>
-#include <libioP.h>
+#include "stdio.h"
+#include "libioP.h"
 
 void
 rewind(fp)
