@@ -45,7 +45,7 @@ static void default_free(void* ptr)
 
 #define SET_STR_JUMPS(STRBUF) \
   (STRBUF)->_jumps = &_IO_str_jumps,\
-  (STRBUF)->_vtable() = NULL;
+  (STRBUF)->_vtable() = builtinbuf_vtable;
 
 istrstream::istrstream(const char *cp, int n)
 {
